*     JLdL 26May06.
*     Copyright (C) 1999-2006 by Jorge L. deLyra.
*     Mail: <delyra@latt.if.usp.br>. Web: "http://latt.if.usp.br".
*     This program may be copied and/or distributed freely. See the
*     _ terms and conditions in the files in the doc/ subdirectory.
*     
*     This routine calculates the sum of the dimensionful version
*     _ of the momentum-space blocked propagator over all the
*     _ momentum modes on the torus, in layers, for a given mass,
*     _ a given even lattice size and a given blocking ratio,
*     _ in four dimensions.
*     
      SUBROUTINE BWIDTH_D4_E (nvert,amass,ratio,width)
*     
*     Double precision everything.
      implicit double precision (a-h,o-z)
*     
*     Define the fixed parameters.
      parameter (mdime=4,mvert=200)
      parameter (mmoms=mvert/2,mlays=mdime*mmoms**2)
*     
*     Storage for the layers, periodic functions and form factors.
      dimension suml(0:mlays),squine(0:mmoms),formf(0:mmoms)
*     
*     Check whether the mass is OK.
      if (amass.eq.0.0d0) then
         write(*,*) 'bwidth_d4_e: ERROR: cannot run for zero mass'
         stop
      end if
*     
*     Check whether nvert is OK.
      if (nvert.ne.2*(nvert/2)) then
         write(*,*) 'bwidth_d4_e: ERROR: nvert must be even'
         stop
      end if
      if (nvert.gt.mvert) then
         write(*,*) 'bwidth_d4_e: ERROR: nvert must be <= ',mvert
         stop
      end if
*     
*     Check whether the ratio is OK.
      if (ratio.lt.1.0d0) then
         write(*,*) 'bwidth_d4_e: ERROR: ratio must be >= 1'
         stop
      end if
*     
*     Redefine the mass.
      amass=0.25d0*amass**2
*     
*     Initialize nvert-dependent quantities.
      nmoms=nvert/2
      nlays=mdime*nmoms**2
      pin=dacos(-1.0d0)/dble(nvert)
      alpha=amass/dble(nvert)**2
*     
*     Initialize ratio-dependent things.
      pir=dacos(-1.0d0)/ratio
*     
*     Store the squared sines in an array.
      do i=0,nmoms
         squine(i)=dsin(dble(i)*pin)**2
      end do
*     
*     Initialize the form factors.
      formf(0)=1.0d0
      fnorm=ratio/dble(nvert)
      do i=1,nmoms
         formf(i)=fnorm*dsin(dble(i)*pir)/dsin(dble(i)*pin)
         formf(i)=formf(i)**2
      end do
*     
*     Initialize accumulators.
      do layer=0,nlays
         suml(layer)=0.0d0
      end do
*     
*     Add the terms into layers.
      do l=0,nmoms
         if (l.eq.0.or.l.eq.nmoms) then
            factl=1.0d0
         else
            factl=2.0d0
         end if
         do k=0,nmoms
            if (k.eq.0.or.k.eq.nmoms) then
               factk=1.0d0
            else
               factk=2.0d0
            end if
            do j=0,nmoms
               if (j.eq.0.or.j.eq.nmoms) then
                  factj=1.0d0
               else
                  factj=2.0d0
               end if
               do i=0,nmoms
                  if (i.eq.0.or.i.eq.nmoms) then
                     facti=1.0d0
                  else
                     facti=2.0d0
                  end if
                  layer=i**2+j**2+k**2+l**2
                  suml(layer)=suml(layer)
     *                 +facti*factj*factk*factl
     *                 *formf(i)*formf(j)*formf(k)*formf(l)
     *                 /(squine(i)+squine(j)
     *                 +squine(k)+squine(l)+alpha)
               end do
            end do
         end do
      end do
*     
*     Add up the layers.
      sum=0.0d0
      do layer=nlays,0,-1
         sum=sum+suml(layer)
      end do
*     
*     Normalize the sum and define the width.
      width=sum/(4.0d0*dble(nvert)**2)
*     
      return
*     
      END
